# -*- perl -*-
#
# Copyright (C) 1996 Ken MacLeod
# See the file COPYING for distribution terms.
#

sub BEGIN {
    # XXX ${srcdir}?
    unshift (@INC, "..");
}

use Text::GDO;
use FileHandle;

($prog = $0) =~ s|.*/||;

if ($ARGV[0] eq "--keep") {
    shift (@ARGV);
    $keep = 1;
}

Text::GDO->init;

test_for_html();

STDOUT->autoflush (1);

foreach $test (@ARGV) {
    require "$test.pl";
}

my ($all_passed) = 1;
foreach $test (sort (keys %{$tests})) {
    next if defined $tests->{$test}{'no-test'};

    if ($keep) {
	if (-d $test) {
	    system "rm -rf $test";
	}
	mkdir ($test, 0777)
	    || die "unable to create directory '$test': $!\n";
    }

    open (LOG, ">$test.log")
	|| die "Can't open '$test.log': $!\n";

    LOG->autoflush (1);

    printf "%-20s   ", $test;
    my (@formats) = sort (keys %{$tests->{$test}{'results'}});
    foreach $format (@formats) {
	printf "%-10s ", $format;
    }
    print "\n";

    my ($passed) = 1;
    unshift (@temp_files, "sroff-in", "sroff-err", "sroff-out", "check-out");
    foreach $key (sort (keys %{$tests->{$test}{'source'}})) {
	my ($test_name) = $key;
	my ($source) = $tests->{$test}{'source'}{$key};

	my ($instance) = new Text::GDO::SGML ('source' => $source)
	    || die "$::prog: can't load source for test \`$test_name', exiting\n";

	my ($specification) = new Text::GDO::Specification ($instance)
	    || die "$::prog: can't load specification for \`$instance->instance_name'\n";

	my ($transform) = Text::GDO::Specification->merge ($specification)
	    || die "$::prog: can't merge specifications\n";

	my ($mapped_instance) = $transform->map_instance ($instance)
	    || die "$::prog: could not map document for test \`$test_name', exiting\n";

	printf "  %-20s ", $test_name;
	foreach $format (@formats) {
	    my ($result) = $tests->{$test}{'results'}{$format}{'expected'}{$key};

	    print LOG "==== $test_name ====\n";
	    print LOG "---- source ----\n$source";
	    print LOG "---- expected result ----\n$result";
	    print LOG "\n" if (substr ($result, -1) ne "\n");

	    my ($output_type) = $tests->{$test}{'results'}{$format}{'output_type'};
	    my ($output_instance) = $mapped_instance->to ($output_type)
		|| die "$::prog: could not format document for test \`$test_name'\n";
	    $sroff_out = $output_instance->scalar;

	    print LOG "---- sroff errors ----\n$sroff_err";
	    print LOG "---- actual result ----\n$sroff_out";

	    if ($sroff_err eq "" && $sroff_out eq $result) {
		&{$tests->{$test}{'results'}{$format}{'check'}};
		$check = &file_data ("check-out");
		print LOG "---- check ----\n$check";
	    }

	    if ($keep) {
		open (OUT, ">$test/$key.$format")
		    || die "$::prog: can't open keep-file \`$test/$key.$format', $!\n";
		print OUT $sroff_out;
		close OUT;
	    }

	    if ($check ne ""
		|| $sroff_err ne "" || $sroff_out ne $result) {
		if ($tests->{$test}{'results'}{$format}{'will_fail'}{$key}) {
		    printf "%-10s ", "(FAILED)";
		    print LOG "**** (FAILED) ****\n\n";
		} else {
		    $passed = 0;
		    printf "%-10s ", "FAILED";
		    print LOG "**** FAILED ****\n\n";
		}
	    } else {
		printf "%-10s ", "Passed";
		print LOG "---- Passed ----\n\n";
	    }
	}

	print "\n";

	unlink (@temp_files);
    }

    close (LOG);

    if ($passed) {
	printf "     %-20s Passed\n", $test;
    } else {
	$all_passed = 0;
	printf "      %-20s FAILED\n", $test;
    }
}

if ($all_passed) {
    exit 0;
} else {
    exit 1;
}

sub file_data {
    my ($file_name) = @_;
    my ($str);

    @ARGV = ($file_name);
    undef $/;
    $str = <>;
    $/ = "\n";

    return $str;
}

sub test_for_html {
    my ($segment);

    open (SGMLS, "| $Text::GDO::NSGMLS -s >check-out 2>&1")
	|| die "$::prog: can't open pipe to \`$Text::GDO::NSGMLS', $!\n";
    print SGMLS <<'EOF';
<!DOCTYPE html PUBLIC "-//IETF//DTD HTML 2.0//EN">
<HTML>
  <HEAD>
    <TITLE></TITLE>
  </HEAD>
  <BODY>
  </BODY>
</HTML>
EOF
    close SGMLS;

    if (-s "check-out") {
	warn "$::prog: error checking for HTML 2.0 DTD, not installed?\n";
	$::test_html = 0;
    } else {
	$::test_html = 1;
    }
};
